@@ -1,7 +1,6 @@
-/* execute.c - run a bc program. */
-
 /*  This file is part of GNU bc.
-    Copyright (C) 1991, 1992, 1993, 1994, 1997 Free Software Foundation, Inc.
+
+    Copyright (C) 1991-1994, 1997, 2006 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,21 +13,23 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; see the file COPYING.  If not, write to
-    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+    along with this program; see the file COPYING.  If not, write to:
+      The Free Software Foundation, Inc.
+      Foundation, Inc.  51 Franklin Street, Fifth Floor,
+      Boston, MA 02110-1301  USA
 
     You may contact the author by:
-       e-mail:  phil@cs.wwu.edu
+       e-mail:  philnelson@acm.org
       us-mail:  Philip A. Nelson
                 Computer Science Department, 9062
                 Western Washington University
                 Bellingham, WA 98226-9062
        
 *************************************************************************/
+/* execute.c - run a bc program. */
 
 #include "bcdefs.h"
 #include <signal.h>
-#include "global.h"
 #include "proto.h"
 
 
@@ -41,8 +42,6 @@ stop_execution (sig)
      int sig;
 {
   had_sigint = TRUE;
-  printf ("\n");
-  rt_error ("interrupted execution");
 }
 
 
@@ -52,11 +51,7 @@ unsigned char
 byte (pc)
      program_counter *pc;
 {
-  int seg, offset;
-    
-  seg = pc->pc_addr >> BC_SEG_LOG;
-  offset = pc->pc_addr++ % BC_SEG_SIZE;
-  return (functions[pc->pc_func].f_body[seg][offset]);
+  return (functions[pc->pc_func].f_body[pc->pc_addr++]);
 }
 
 
@@ -81,16 +76,17 @@ execute ()
   pc.pc_func = 0;
   pc.pc_addr = 0;
   runtime_error = FALSE;
-  init_num (&temp_num);
+  bc_init_num (&temp_num);
 
   /* Set up the interrupt mechanism for an interactive session. */
   if (interactive)
     {
       signal (SIGINT, stop_execution);
-      had_sigint = FALSE;
     }
    
-  while (pc.pc_addr < functions[pc.pc_func].f_code_size && !runtime_error)
+  had_sigint = FALSE;
+  while (pc.pc_addr < functions[pc.pc_func].f_code_size
+	 && !runtime_error && !had_sigint)
     {
       inst = byte(&pc);
 
@@ -105,11 +101,12 @@ execute ()
 	    depth = 1;
 	    while (temp != NULL)
 	      {
-		printf ("   %d = ", depth);
-		out_num (temp->s_num, 10, out_char);
+		printf ("  %d = ", depth);
+		bc_out_num (temp->s_num, 10, out_char, std_only);
 		depth++;
 		temp = temp->s_next;
 	      }
+	    out_char ('\n');
 	  }
       }
 #endif
@@ -120,13 +117,13 @@ execute ()
       case 'A' : /* increment array variable (Add one). */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f) << 8) + byte(&pc);
 	incr_array (var_name);
 	break;
 
       case 'B' : /* Branch to a label if TOS != 0. Remove value on TOS. */
       case 'Z' : /* Branch to a label if TOS == 0. Remove value on TOS. */
-	c_code = !is_zero (ex_stack->s_num);
+	c_code = !bc_is_zero (ex_stack->s_num);
 	pop ();
       case 'J' : /* Jump to a label. */
 	label_num = byte(&pc);  /* Low order bits first. */
@@ -145,7 +142,7 @@ execute ()
 	/* Get the function number. */
 	new_func = byte(&pc);
 	if ((new_func & 0x80) != 0) 
-	  new_func = ((new_func << 8) & 0x7f) + byte(&pc);
+	  new_func = ((new_func & 0x7f) << 8) + byte(&pc);
 
 	/* Check to make sure it is defined. */
 	if (!functions[new_func].f_defined)
@@ -192,14 +189,14 @@ execute ()
       case 'L' : /* load array variable */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f) << 8) + byte(&pc);
 	load_array (var_name);
 	break;
 
       case 'M' : /* decrement array variable (Minus!) */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f) << 8) + byte(&pc);
 	decr_array (var_name);
 	break;
 
@@ -245,18 +242,18 @@ execute ()
       case 'S' : /* store array variable */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f ) << 8) + byte(&pc);
 	store_array (var_name);
 	break;
 
       case 'T' : /* Test tos for zero */
-	c_code = is_zero (ex_stack->s_num);
+	c_code = bc_is_zero (ex_stack->s_num);
 	assign (c_code);
 	break;
 
       case 'W' : /* Write the value on the top of the stack. */
       case 'P' : /* Write the value on the top of the stack.  No newline. */
-	out_num (ex_stack->s_num, o_base, out_char);
+	bc_out_num (ex_stack->s_num, o_base, out_char, std_only);
 	if (inst == 'W') out_char ('\n');
 	store_var (4);  /* Special variable "last". */
 	fflush (stdout);
@@ -273,14 +270,14 @@ execute ()
 	  if (ex_stack->s_num->n_len == 1 &&
 	      ex_stack->s_num->n_scale != 0 &&
 	      ex_stack->s_num->n_value[0] == 0 )
-	    int2num (&ex_stack->s_num, ex_stack->s_num->n_scale);
+	    bc_int2num (&ex_stack->s_num, ex_stack->s_num->n_scale);
 	  else
-	    int2num (&ex_stack->s_num, ex_stack->s_num->n_len
+	    bc_int2num (&ex_stack->s_num, ex_stack->s_num->n_len
 		     + ex_stack->s_num->n_scale);
 	  break;
 		
 	case 'S':  /* Scale function. */ 
-	  int2num (&ex_stack->s_num, ex_stack->s_num->n_scale);
+	  bc_int2num (&ex_stack->s_num, ex_stack->s_num->n_scale);
 	  break;
 
 	case 'R':  /* Square Root function. */
@@ -291,13 +288,18 @@ execute ()
 	case 'I': /* Read function. */
 	  push_constant (input_char, i_base);
 	  break;
+
+	case 'X': /* Random function. */
+	  push_copy (_zero_);
+	  bc_int2num (&ex_stack->s_num, random());
+	  break;
 	}
 	break;
 
       case 'd' : /* Decrement number */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f) << 8) + byte(&pc);
 	decr_var (var_name);
 	break;
       
@@ -307,14 +309,14 @@ execute ()
       case 'i' : /* increment number */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f) << 8) + byte(&pc);
 	incr_var (var_name);
 	break;
 
       case 'l' : /* load variable */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f) << 8) + byte(&pc);
 	load_var (var_name);
 	break;
 
@@ -329,7 +331,7 @@ execute ()
       case 's' : /* store variable */
 	var_name = byte(&pc);
 	if ((var_name & 0x80) != 0)
-	  var_name = ((var_name << 8) & 0x7f) + byte(&pc);
+	  var_name = ((var_name & 0x7f) << 8) + byte(&pc);
 	store_var (var_name);
 	break;
 
@@ -350,20 +352,20 @@ execute ()
 	push_copy (_zero_);
 	break;
 
-      case '1' : /* Load Constant 0. */
+      case '1' : /* Load Constant 1. */
 	push_copy (_one_);
 	break;
 
       case '!' : /* Negate the boolean value on top of the stack. */
-	c_code = is_zero (ex_stack->s_num);
+	c_code = bc_is_zero (ex_stack->s_num);
 	assign (c_code);
 	break;
 
       case '&' : /* compare greater than */
 	if (check_stack(2))
 	  {
-	    c_code = !is_zero (ex_stack->s_next->s_num)
-	      && !is_zero (ex_stack->s_num);
+	    c_code = !bc_is_zero (ex_stack->s_next->s_num)
+	      && !bc_is_zero (ex_stack->s_num);
 	    pop ();
 	    assign (c_code);
 	  }
@@ -372,8 +374,8 @@ execute ()
       case '|' : /* compare greater than */
 	if (check_stack(2))
 	  {
-	    c_code = !is_zero (ex_stack->s_next->s_num)
-	      || !is_zero (ex_stack->s_num);
+	    c_code = !bc_is_zero (ex_stack->s_next->s_num)
+	      || !bc_is_zero (ex_stack->s_num);
 	    pop ();
 	    assign (c_code);
 	  }
@@ -386,7 +388,7 @@ execute ()
 	    pop();
 	    pop();
 	    push_num (temp_num);
-	    init_num (&temp_num);
+	    bc_init_num (&temp_num);
 	  }
 	break;
 
@@ -397,7 +399,7 @@ execute ()
 	    pop();
 	    pop();
 	    push_num (temp_num);
-	    init_num (&temp_num);
+	    bc_init_num (&temp_num);
 	  }
 	break;
 
@@ -409,7 +411,7 @@ execute ()
 	    pop();
 	    pop();
 	    push_num (temp_num);
-	    init_num (&temp_num);
+	    bc_init_num (&temp_num);
 	  }
 	break;
 
@@ -422,7 +424,7 @@ execute ()
 		pop();
 		pop();
 		push_num (temp_num);
-		init_num (&temp_num);
+		bc_init_num (&temp_num);
 	      }
 	    else
 	      rt_error ("Divide by zero");
@@ -432,7 +434,7 @@ execute ()
       case '%' : /* remainder */
 	if (check_stack(2))
 	  {
-	    if (is_zero (ex_stack->s_num))
+	    if (bc_is_zero (ex_stack->s_num))
 	      rt_error ("Modulo by zero");
 	    else
 	      {
@@ -441,7 +443,7 @@ execute ()
 		pop();
 		pop();
 		push_num (temp_num);
-		init_num (&temp_num);
+		bc_init_num (&temp_num);
 	      }
 	  }
 	break;
@@ -451,12 +453,12 @@ execute ()
 	  {
 	    bc_raise (ex_stack->s_next->s_num,
 		      ex_stack->s_num, &temp_num, scale);
-	    if (is_zero (ex_stack->s_next->s_num) && is_neg (ex_stack->s_num))
+	    if (bc_is_zero (ex_stack->s_next->s_num) && bc_is_neg (ex_stack->s_num))
 	      rt_error ("divide by zero");
 	    pop();
 	    pop();
 	    push_num (temp_num);
-	    init_num (&temp_num);
+	    bc_init_num (&temp_num);
 	  }
 	break;
 
@@ -543,7 +545,7 @@ execute ()
     {
       signal (SIGINT, use_quit);
       if (had_sigint)
-	printf ("Interruption completed.\n");
+	printf ("\ninterrupted execution.\n");
     }
 }
 
@@ -551,20 +553,20 @@ execute ()
 /* Prog_char gets another byte from the program.  It is used for
    conversion of text constants in the code to numbers. */
 
-char
+int
 prog_char ()
 {
-  return byte(&pc);
+  return (int) byte(&pc);
 }
 
 
 /* Read a character from the standard input.  This function is used
    by the "read" function. */
 
-char
+int
 input_char ()
 {
-  char in_ch;
+  int in_ch;
   
   /* Get a character from the standard input for the read function. */
   in_ch = getchar();
@@ -573,8 +575,10 @@ input_char ()
   if (in_ch == '\\')
     {
       in_ch = getchar();
-      if (in_ch == '\n')
-	in_ch = getchar();
+      if (in_ch == '\n') {
+	  in_ch = getchar();
+	  out_col = 0;  /* Saw a new line */
+	}
     }
 
   /* Classify and preprocess the input character. */
@@ -599,23 +603,23 @@ input_char ()
 
 void
 push_constant (in_char, conv_base)
-   char (*in_char)(VOID);
+   int (*in_char)(VOID);
    int conv_base;
 {
   int digits;
   bc_num build, temp, result, mult, divisor;
-  char  in_ch, first_ch;
+  int   in_ch, first_ch;
   char  negative;
 
   /* Initialize all bc numbers */
-  init_num (&temp);
-  init_num (&result);
-  init_num (&mult);
-  build = copy_num (_zero_);
+  bc_init_num (&temp);
+  bc_init_num (&result);
+  bc_init_num (&mult);
+  build = bc_copy_num (_zero_);
   negative = FALSE;
 
   /* The conversion base. */
-  int2num (&mult, conv_base);
+  bc_int2num (&mult, conv_base);
   
   /* Get things ready. */
   in_ch = in_char();
@@ -638,7 +642,7 @@ push_constant (in_char, conv_base)
       in_ch = in_char();
       if (in_ch < 16 && first_ch >= conv_base)
 	first_ch = conv_base - 1;
-      int2num (&build, (int) first_ch);
+      bc_int2num (&build, (int) first_ch);
     }
 
   /* Convert the integer part. */
@@ -646,7 +650,7 @@ push_constant (in_char, conv_base)
     {
       if (in_ch < 16 && in_ch >= conv_base) in_ch = conv_base-1;
       bc_multiply (build, mult, &result, 0);
-      int2num (&temp, (int) in_ch);
+      bc_int2num (&temp, (int) in_ch);
       bc_add (result, temp, &build, 0);
       in_ch = in_char();
     }
@@ -654,15 +658,15 @@ push_constant (in_char, conv_base)
     {
       in_ch = in_char();
       if (in_ch >= conv_base) in_ch = conv_base-1;
-      free_num (&result);
-      free_num (&temp);
-      divisor = copy_num (_one_);
-      result = copy_num (_zero_);
+      bc_free_num (&result);
+      bc_free_num (&temp);
+      divisor = bc_copy_num (_one_);
+      result = bc_copy_num (_zero_);
       digits = 0;
       while (in_ch < 16)
 	{
 	  bc_multiply (result, mult, &result, 0);
-	  int2num (&temp, (int) in_ch);
+	  bc_int2num (&temp, (int) in_ch);
 	  bc_add (result, temp, &result, 0);
 	  bc_multiply (divisor, mult, &divisor, 0);
 	  digits++;
@@ -678,9 +682,9 @@ push_constant (in_char, conv_base)
     bc_sub (_zero_, build, &build, 0);
 
   push_num (build);
-  free_num (&temp);
-  free_num (&result);
-  free_num (&mult);
+  bc_free_num (&temp);
+  bc_free_num (&result);
+  bc_free_num (&mult);
 }
 
 
@@ -696,7 +700,7 @@ push_b10_const (pc)
   bc_num build;
   program_counter look_pc;
   int kdigits, kscale;
-  char inchar;
+  unsigned char inchar;
   char *ptr;
   
   /* Count the digits and get things ready. */
@@ -738,8 +742,8 @@ push_b10_const (pc)
     }
     if (inchar > 9)
       {
-	init_num (&build);
-	int2num (&build, inchar);
+	bc_init_num (&build);
+	bc_int2num (&build, inchar);
 	push_num (build);
 	inchar = byte(pc);
 	return;
@@ -749,23 +753,25 @@ push_b10_const (pc)
   /* Build the new number. */
   if (kdigits == 0)
     {
-      build = new_num (1,kscale);
+      build = bc_new_num (1,kscale);
       ptr = build->n_value;
       *ptr++ = 0;
     }
   else
     {
-      build = new_num (kdigits,kscale);
+      build = bc_new_num (kdigits,kscale);
       ptr = build->n_value;
     }
 
   while (inchar != ':')
     {
       if (inchar != '.')
-	if (inchar > 9)
-	  *ptr++ = 9;
-	else
-	  *ptr++ = inchar;
+	{
+	  if (inchar > 9)
+	    *ptr++ = 9;
+	  else
+	    *ptr++ = inchar;
+	}
       inchar = byte(pc);
     }
   push_num (build);
@@ -778,9 +784,10 @@ void
 assign (c_code)
      char c_code;
 {
-  free_num (&ex_stack->s_num);
+  bc_free_num (&ex_stack->s_num);
   if (c_code)
-    ex_stack->s_num = copy_num (_one_);
+    ex_stack->s_num = bc_copy_num (_one_);
   else
-    ex_stack->s_num = copy_num (_zero_);
+    ex_stack->s_num = bc_copy_num (_zero_);
 }
+
